package sun.awt.image.codec;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.image.DataBufferInt;
import java.awt.image.DataBufferByte;
import java.awt.Point;
import java.awt.image.DirectColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.color.ColorSpace;
import com.sun.image.codec.jpeg.ImageFormatException;
import java.io.IOException;
import java.awt.image.Raster;
import java.awt.image.ColorModel;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.InputStream;
import com.sun.image.codec.jpeg.JPEGDecodeParam;
import com.sun.image.codec.jpeg.JPEGImageDecoder;

public class JPEGImageDecoderImpl implements JPEGImageDecoder
{
    private static final Class InputStreamClass;
    private JPEGDecodeParam param;
    private InputStream input;
    private WritableRaster aRas;
    private BufferedImage aBufImg;
    private ColorModel cm;
    private boolean unpack;
    private boolean flip;
    
    public JPEGImageDecoderImpl(InputStream input) {
        this.param = null;
        this.input = null;
        this.aRas = null;
        this.aBufImg = null;
        this.cm = null;
        this.unpack = false;
        this.flip = false;
        if (input == null) {
            throw new IllegalArgumentException("InputStream is null.");
        }
        this.input = input;
        this.initDecoder(JPEGImageDecoderImpl.InputStreamClass);
        return;
    }
    
    public JPEGImageDecoderImpl(InputStream inputStream, JPEGDecodeParam jpegDecodeParam) {
        this(inputStream);
        this.setJPEGDecodeParam(jpegDecodeParam);
    }
    
    public JPEGDecodeParam getJPEGDecodeParam() {
        if (this.param != null) {
            return (JPEGDecodeParam)this.param.clone();
        }
        return null;
    }
    
    public void setJPEGDecodeParam(JPEGDecodeParam jpegDecodeParam) {
        this.param = (JPEGDecodeParam)jpegDecodeParam.clone();
    }
    
    public synchronized InputStream getInputStream() {
        return this.input;
    }
    
    public synchronized Raster decodeAsRaster() throws ImageFormatException {
        try {
            this.param = this.readJPEGStream(this.input, this.param, false);
        }
        catch (IOException obj) {
            System.out.println("Can't open input Stream" + obj);
            obj.printStackTrace();
        }
        return this.aRas;
    }
    
    public synchronized BufferedImage decodeAsBufferedImage() throws ImageFormatException {
        try {
            this.param = this.readJPEGStream(this.input, this.param, true);
        }
        catch (IOException obj) {
            System.out.println("Can't open input Stream" + obj);
            obj.printStackTrace();
        }
        return this.aBufImg;
    }
    
    private native void initDecoder(Class p0);
    
    private synchronized native JPEGDecodeParam readJPEGStream(InputStream p0, JPEGDecodeParam p1, boolean p2) throws IOException, ImageFormatException;
    
    private void readTables() throws IOException {
        try {
            this.param = this.readJPEGStream(this.input, null, false);
        }
        catch (ImageFormatException ex) {
            ex.printStackTrace();
        }
    }
    
    private int getDecodedColorModel(int n, boolean b) throws ImageFormatException {
        int[] bits = { 8 };
        int[] bits2 = { 8, 8, 8 };
        int[] bits3 = { 8, 8, 8, 8 };
        this.cm = null;
        this.unpack = false;
        this.flip = false;
        if (!b) {
            return n;
        }
        switch (n) {
            case 1: {
                this.cm = new ComponentColorModel(ColorSpace.getInstance(1003), bits, false, false, 1, 0);
                return n;
            }
            case 5: {
                this.cm = new ComponentColorModel(ColorSpace.getInstance(1002), bits2, false, false, 1, 0);
                return n;
            }
            case 10: {
                this.cm = new ComponentColorModel(ColorSpace.getInstance(1002), bits3, true, false, 3, 0);
                return n;
            }
            case 2:
            case 3: {
                this.unpack = true;
                this.cm = new DirectColorModel(24, 16711680, 65280, 255);
                return 2;
            }
            case 8:
            case 9: {
                this.flip = true;
            }
            case 6:
            case 7: {
                this.unpack = true;
                this.cm = new DirectColorModel(ColorSpace.getInstance(1000), 32, 16711680, 65280, 255, -16777216, false, 3);
                return 6;
            }
            default: {
                throw new ImageFormatException("Can't construct a BufferedImage for given COLOR_ID");
            }
        }
    }
    
    private Object allocateDataBuffer(int w, int h, int bands) {
        int[] array;
        if (this.unpack) {
            if (bands == 3) {
                this.aRas = Raster.createPackedRaster(3, w, h, new int[] { 16711680, 65280, 255 }, new Point(0, 0));
            }
            else if (bands == 4) {
                this.aRas = Raster.createPackedRaster(3, w, h, new int[] { 16711680, 65280, 255, -16777216 }, new Point(0, 0));
            }
            else {
                throw new ImageFormatException("Can't unpack with anything other than 3 or 4 components");
            }
            array = ((DataBufferInt)this.aRas.getDataBuffer()).getData();
        }
        else {
            this.aRas = Raster.createInterleavedRaster(0, w, h, bands, new Point(0, 0));
            array = ((DataBufferByte)this.aRas.getDataBuffer()).getData();
        }
        if (this.cm != null) {
            this.aBufImg = new BufferedImage(this.cm, this.aRas, true, null);
            return array;
        }
        return array;
    }
    
    static {
        InputStreamClass = InputStream.class;
        AccessController.doPrivileged(new PrivilegedAction<Void>() {
            @Override
            public Void run() {
                System.loadLibrary("jpeg");
                return null;
            }
        });
    }
}
